@@ -13,6 +13,7 @@ import os, math
 import gtk
 from gtk import gdk
 from gettext import gettext as _
+from bisect import bisect_left
 
 import lib.document
 from lib import backgroundsurface, command, helpers, layer
@@ -36,9 +37,12 @@ class Document(object):
         self.zoomlevel_values = [1.0/8, 2.0/11, 0.25, 1.0/3, 0.50, 2.0/3,  # micro
                                  1.0, 1.5, 2.0, 3.0, 4.0, 5.5, 8.0,        # normal
                                  11.0, 16.0, 23.0, 32.0, 45.0, 64.0]       # macro
+                                 # keep sorted for bisect
 
         default_zoom = self.app.preferences['view.default_zoom']
-        self.zoomlevel = self.zoomlevel_values.index(default_zoom)
+        self.zoomlevel = min(bisect_left(self.zoomlevel_values, default_zoom),
+                             len(self.zoomlevel_values) - 1)
+        default_zoom = self.zoomlevel_values[self.zoomlevel]
         self.tdw.scale = default_zoom
         self.tdw.zoom_min = min(self.zoomlevel_values)
         self.tdw.zoom_max = max(self.zoomlevel_values)
@@ -101,8 +105,8 @@ class Document(object):
             ('ResetView',   gtk.STOCK_ZOOM_100, _('Reset (Zoom, Rotation, Mirror)'), 'F12', None, self.reset_view_cb),
             ('ZoomIn',       gtk.STOCK_ZOOM_IN, _('Zoom In (at cursor)'), 'period', None, self.zoom_cb),
             ('ZoomOut',      gtk.STOCK_ZOOM_OUT, _('Zoom Out'), 'comma', None, self.zoom_cb),
-            ('RotateLeft',   None, _('Rotate Counterclockwise'), None, None, self.rotate_cb),
-            ('RotateRight',  None, _('Rotate Clockwise'), None, None, self.rotate_cb),
+            ('RotateLeft',   None, _('Rotate Counterclockwise'), '<control>Left', None, self.rotate_cb),
+            ('RotateRight',  None, _('Rotate Clockwise'), '<control>Right', None, self.rotate_cb),
             ('MirrorHorizontal', None, _('Mirror Horizontal'), 'i', None, self.mirror_horizontal_cb),
             ('MirrorVertical', None, _('Mirror Vertical'), 'u', None, self.mirror_vertical_cb),
             ('SoloLayer',    None, _('Layer Solo'), 'Home', None, self.solo_layer_cb), # TODO: make toggle action
@@ -474,8 +478,11 @@ class Document(object):
         self.tdw.set_zoom(z)
 
     def rotate(self, command):
-        if   command == 'RotateRight': self.tdw.rotate(+2*math.pi/14)
-        elif command == 'RotateLeft' : self.tdw.rotate(-2*math.pi/14)
+        # Allows easy and quick rotation to 45/90/180 degrees
+        rotation_step = 2*math.pi/16
+
+        if   command == 'RotateRight': self.tdw.rotate(+rotation_step)
+        elif command == 'RotateLeft' : self.tdw.rotate(-rotation_step)
         else: assert 0
 
     def zoom_cb(self, action):
