@@ -242,19 +242,22 @@ class FileHandler(object):
         except document.SaveLoadError, e:
             self.app.message_dialog(str(e),type=gtk.MESSAGE_ERROR)
         else:
+            file_location = None
             if not export:
-                self.filename = os.path.abspath(filename)
+                file_location = self.filename = os.path.abspath(filename)
                 print 'Saved to', self.filename
                 gtk.recent_manager_get_default().add_full(helpers.filename2uri(self.filename),
                         {
                             'app_name': 'mypaint',
-                            'app_exec': sys.argv[0],
+                            'app_exec': sys.argv[0].encode('utf-8'), # on windows we manipulated sys.argv to be unicode
                             # todo: get mime_type
                             'mime_type': 'application/octet-stream'
                         }
                 )
             else:
-                print 'Exported to', os.path.abspath(filename)
+                file_location = os.path.abspath(filename)
+                print 'Exported to', os.path.abspath(file_location)
+            helpers.save_freedesktop_thumbnail(None, file_location)
 
     def update_preview_cb(self, file_chooser, preview):
         filename = file_chooser.get_preview_filename()
@@ -424,7 +427,7 @@ class FileHandler(object):
 
     def get_scrap_prefix(self):
         prefix = self.app.preferences['saving.scrap_prefix']
-        prefix = os.path.expanduser(prefix.decode('utf-8'))
+        prefix = helpers.expanduser_unicode(prefix.decode('utf-8'))
         prefix = os.path.abspath(prefix)
         if os.path.isdir(prefix):
             if not prefix.endswith(os.path.sep):
