@@ -11,6 +11,7 @@ pygtk.require('2.0')
 import gtk
 required = (2, 16, 0)
 assert gtk.ver >= required, 'You need to upgrade PyGTK, at least version %d.%d.%d is required.' % required
+import gobject
 
 from gui import application
 from optparse import OptionParser
@@ -26,18 +27,29 @@ def main(datapath, confpath):
                     help='append python stdout and stderr to FILE')
     parser.add_option('-t', '--trace', action="store_true",
                     help='print all exectued python statements')
+    parser.add_option('-f', '--fullscreen', action="store_true",
+                    help='start in fullscreen mode')
     options, args = parser.parse_args()
 
+    if sys.platform != 'win32':
+        args = [s.decode(sys.getfilesystemencoding()) for s in args]
+
     if options.logfile:
         print 'Python prints are redirected to', options.logfile, 'after this one.'
         sys.stdout = sys.stderr = open(options.logfile, 'a', 1)
         print '--- mypaint log %s ---' % time.strftime('%Y-%m-%d %H:%M:%S')
 
-    confpath = options.config.decode(sys.getfilesystemencoding())
+    if sys.platform != 'win32':
+        confpath = options.config.decode(sys.getfilesystemencoding())
 
     def run():
         print 'confpath =', options.config
+
         app = application.Application(datapath, confpath, args)
+        if options.fullscreen:
+            def f():
+                app.drawWindow.fullscreen_cb()
+            gobject.idle_add(f)
 
         # Recent gtk versions don't allow changing those menu shortcuts by
         # default. <rant>Sigh. This very useful feature used to be the
