@@ -9,6 +9,7 @@
 import gtk, gobject, cairo, random
 gdk = gtk.gdk
 from math import floor, ceil, pi, log
+from numpy import isfinite
 
 from lib import helpers, tiledsurface, pixbufsurface
 import cursor
@@ -116,6 +117,14 @@ class TiledDrawWidget(gtk.DrawingArea):
             dy = old_size[1] - new_size[1]
             self.scroll(dx/2, dy/2)
 
+    def device_used(self, device):
+        """Tell the TDW about a device being used."""
+        if device == self.last_event_device:
+            return
+        for func in self.device_observers:
+            func(self.last_event_device, device)
+        self.last_event_device = device
+
     def motion_notify_cb(self, widget, event, button1_pressed=None):
         if not self.is_sensitive:
             return
@@ -126,13 +135,10 @@ class TiledDrawWidget(gtk.DrawingArea):
             dy = event.y - self.last_event_y
         else:
             dtime = None
-        if event.device != self.last_event_device:
-            for f in self.device_observers:
-                f(self.last_event_device, event.device)
+        self.device_used(event.device)
         self.last_event_x = event.x
         self.last_event_y = event.y
         self.last_event_time = event.time
-        self.last_event_device = event.device
         if dtime is None:
             return
 
@@ -145,12 +151,12 @@ class TiledDrawWidget(gtk.DrawingArea):
         
         pressure = event.get_axis(gdk.AXIS_PRESSURE)
 
-        if pressure is not None and (pressure > 1.0 or pressure < 0.0):
+        if pressure is not None and (pressure > 1.0 or pressure < 0.0 or not isfinite(pressure)):
             if event.device.name not in self.bad_devices:
                 print 'WARNING: device "%s" is reporting bad pressure %+f' % (event.device.name, pressure)
                 self.bad_devices.append(event.device.name)
-            if pressure > 1000.0 or pressure < -1000.0:
-                # infinity: use button state (instead of clamping in brush.hpp)
+            if not isfinite(pressure):
+                # infinity/nan: use button state (instead of clamping in brush.hpp)
                 # https://gna.org/bugs/?14709
                 pressure = None
 
@@ -167,7 +173,10 @@ class TiledDrawWidget(gtk.DrawingArea):
 
         xtilt = event.get_axis(gdk.AXIS_XTILT)
         ytilt = event.get_axis(gdk.AXIS_YTILT)
-        if xtilt is None or ytilt is None:
+        # Check whether tilt is present.  For some tablets without
+        # tilt support GTK reports a tilt axis with value nan, instead
+        # of None.  https://gna.org/bugs/?17084
+        if xtilt is None or ytilt is None or not isfinite(xtilt+ytilt):
             xtilt = 0.0
             ytilt = 0.0
         
@@ -501,10 +510,12 @@ class TiledDrawWidget(gtk.DrawingArea):
         self.rotozoom_with_center(f, at_pointer=True)
 
     def rotate(self, angle_step):
+        if self.mirrored: angle_step = -angle_step
         def f(): self.rotation += angle_step
         self.rotozoom_with_center(f)
 
     def set_rotation(self, angle):
+        if self.mirrored: angle = -angle
         def f(): self.rotation = angle
         self.rotozoom_with_center(f)
 
