@@ -7,10 +7,10 @@
  * (at your option) any later version.
  */
 
+const int size = 256;
+
 class ColorChanger {
 public:
-  static const int size = 256;
-
   float brush_h, brush_s, brush_v;
   void set_brush_color(float h, float s, float v)
   {
@@ -19,6 +19,11 @@ public:
     brush_v = v;
   }
 
+  int get_size() 
+  {
+    return size;
+  }
+
 #ifndef SWIG
 
   struct PrecalcData {
